# from . import sendmsg
import time

__all__ = ['n2u', 'A', 'mysql_helper', 'NFLError', 'mysql_alter', 'rmb2u', 'exec_time']

name = "fengluU"


class A(object):
    def __init__(self):
        print('A')

    def __str__(self):
        return 'A cls'


class NFLError(Exception):
    pass


def exec_time(func):
    def decorator(*args, **kwargs):
        start = time.time()
        ret = func(*args, **kwargs)
        end = time.time()
        print('function %s\'s execute_time: %ss' % (func.__name__, end - start))
        return ret

    return decorator


if __name__ == '__main__':
    print('-----A has been defined--------')
# print('__name__ is %s' % __name__)



#! /usr/bin/python
# encoding=utf-8
# Created by Fenglu Niu on 2020/9/2 16:05

from fengluU.mysql_helper import *


def gen_ddl(tables_test, tables_prod):
    ddl_sqls_temp = []
    diffs_test = [t for t in tables_test if t not in tables_prod]
    diffs_prod = [t for t in tables_prod if t not in tables_test]
    ddl_sqls_temp.extend(ddl_create(diffs_test))
    ddl_sqls_temp.extend(ddl_drop(diffs_prod))
    return ddl_sqls_temp
    pass


def ddl_create(diffs_test):
    ddl_create_sqls = []
    for table in diffs_test:
        ddl = helper_test.get_table_ddl(table=table)
        ddl = ddl.replace('CREATE TABLE', 'CREATE TABLE IF NOT EXISTS') + '# 已执行'
        helper_prod.exec_ddl(ddl=ddl)
        ddl_create_sqls.append(ddl)
    return ddl_create_sqls
    pass


def ddl_drop(diffs_prod):
    ddl_drop_sqls = []
    for table in diffs_prod:
        ddl_drop_sql = f'DROP TABLE IF EXISTS {table}'
        ddl_drop_sqls.append(ddl_drop_sql)
    return ddl_drop_sqls
    pass


def gen_alter(table: str = None):
    alter_sqls_temp = []
    fields_prod = helper_prod.get_table_fields(table=table)
    fields_test = helper_test.get_table_fields(table=table)
    alter_sqls_temp.extend(alter(table, fields_prod, fields_test))
    alter_sqls_temp.extend(alter_modify(table, fields_prod, fields_test))
    return alter_sqls_temp


def alter_add(table, diffs):
    alter_add_sqls = []
    for diff in diffs:
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_add_sql = f'ALTER TABLE {table} ADD {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_add_sqls.append(alter_add_sql)
    return alter_add_sqls


def alter_change(table, diffs):
    alter_add_sqls = []
    for diff in diffs:
        old = diff.get('old')
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_add_sql = f'ALTER TABLE {table} CHANGE {old} {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_add_sqls.append(alter_add_sql)
    return alter_add_sqls


def alter_drop(table, diffs):
    alter_drop_sqls = []
    for diff in diffs:
        key = diff.get('key')
        alter_drop_sql = f'ALTER TABLE {table} drop {key}'
        alter_drop_sqls.append(alter_drop_sql)
    return alter_drop_sqls


def interactive(table, diffs, diffs_old):
    keys = [row.get('key') for row in diffs_old]
    diffs_add = []
    diffs_change = []
    print('#\033[31m', end='')
    print('\n#接下来进入自主确认改名字段:')
    print('\033[0m', end='')
    print('#表名：', end='')
    print('\033[1;31m', end='')
    print(f'【{table}】', end='')
    print('\033[0m', end='')
    print(f',待确认字段{keys}')
    for i, diff in enumerate(diffs_old):
        prompt = ', '.join([f'\033[1;31m{i}\033[0m)、{e.get("key")}' for i, e in enumerate(diffs)])
        user_input = int(input(f'#[\033[31m{diff.get("key")}\033[0m]要改成哪个呢? {prompt}: '))
        while user_input not in range(len(diffs)):
            user_input = int(input(f'#请您不要玩花样，就输入提示中有的数字!{prompt}: '))
        diff_change = diffs.pop(user_input)
        diff_change.setdefault('old', diff.get('key'))
        diffs_change.append(diff_change)
    diffs_add.extend(diffs)
    return diffs_add, diffs_change


def alter(table, fields_prod, fields_test):
    alter_sqls = []
    diffs = [f for f in fields_test if f.get('key') not in [f.get('key') for f in fields_prod]]
    diffs_old = [f for f in fields_prod if f.get('key') not in [f.get('key') for f in fields_test]]
    delta = len(fields_test) - len(fields_prod)
    if len(diffs) == delta:
        alter_sqls.extend(alter_add(table, diffs))
    elif len(diffs) == 1:
        diffs[0].setdefault('old', diffs_old[0].get('key'))
        alter_sqls.extend(alter_change(table, diffs))
    elif delta < 0:
        alter_sqls.extend(alter_drop(table, diffs_old))
    else:
        diffs_add, diffs_change = interactive(table, diffs, diffs_old)
        alter_sqls.extend(alter_add(table, diffs_add))
        alter_sqls.extend(alter_change(table, diffs_change))
    return alter_sqls


def alter_modify(table, fields_old, fields_new):
    alter_modify_sqls = []
    diffs = [f for f in fields_new if f.get('key') in [f1.get('key') for f1 in fields_old if
                                                       f1.get('type') != f.get('type') or f1.get('null') != f.get(
                                                           'null') or f1.get('default') != f.get('default')]]
    for diff in diffs:
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_modify_sql = f'ALTER TABLE {table} MODIFY {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_modify_sqls.append(alter_modify_sql)
    return alter_modify_sqls


config_prod = {
    'user': 'king',
    'password': 'king001',
    'host': '127.0.0.1',
    'database': 'queen',
    'charset': 'utf8',
    'raise_on_warnings': True,
}

config_test = {
    'user': 'king',
    'password': 'king001',
    'host': '127.0.0.1',
    'database': 'king',
    'charset': 'utf8',
    'raise_on_warnings': True,
}

if __name__ == '__main__':
    helper_prod = MySQLHelper()
    helper_prod.set_config(**config_prod)
    helper_test = MySQLHelper()
    helper_test.set_config(**config_test)
    tables_test = helper_test.show_tables()
    tables_prod = helper_prod.show_tables()
    ddl_sqls = gen_ddl(tables_test, tables_prod)
    if ddl_sqls and len(ddl_sqls) > 0:
        print(f'\n#\033[32m生成的DDL如下:\033[0m')
        for sql in ddl_sqls:
            print(sql)

    for table in tables_test:
        alter_sqls = gen_alter(table)
        if alter_sqls and len(alter_sqls) > 0:
            print(f'\n#\033[32m表{table}生成ALTER SQL如下:\033[0m')
            for sql in alter_sqls:
                print(sql)
    pass



#! /usr/bin/python
# encoding=utf-8
# Created by Fenglu Niu on 2021/12/15 16:35
from fengluU import NFLError
import regex

__all__ = ['rmb2upper']


class RMB2U(object):
    __instance = None
    __is_first = True

    def __new__(cls):
        if not cls.__instance:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def __init__(self):
        if self.__is_first:
            RMB2U.__is_first = False
            self.__RMB_UNIT = ['万', '亿']
            self.__RMB_NUM = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖']

    def append_uint(self, unit: str):
        self.__RMB_UNIT.append(unit)

    def pop_unit(self):
        self.__RMB_UNIT.pop()

    def rmb2upper(self, num: str) -> str:
        """将输入的数字按照人民币大写的书写方式返回(支持16位)
        例如：
            123456789\n
            壹亿贰仟叁佰肆拾伍万陆仟柒佰捌拾玖元整\n

        :param num: 人民币小写
        :return: 返回人民币大写的形式
        """

        def cal_unit(num_index: int) -> str:
            """计算对应编号数字的单位

            :param num_index: 数字的编号，指的是数字中的某一位数字在该数字中从右往左数的编号，编号从0开始\n
            :return: 返回指定编号位数字的单位
            """
            if num_index == 0:  # 如果数字为个位，没有单位
                return ''
            remain = num_index % 4
            if remain > 0:
                if remain == 1:
                    return '拾'
                elif remain == 2:
                    return '佰'
                elif remain == 3:
                    return '仟'
            else:
                quotient_4 = num_index // 4
                if quotient_4 % 2 != 0:
                    return '万'
                else:
                    bin_quotient_4 = bin(quotient_4)
                    c1_bin_quotient_4 = bin_quotient_4.split('b')[1].count('1')
                    c0_bin_quotient_4 = bin_quotient_4.split('b')[1].count('0')
                    if c1_bin_quotient_4 == 1:
                        return self.__RMB_UNIT[c0_bin_quotient_4]
                    else:
                        for check_index in range(1, len(self.__RMB_UNIT)):
                            if quotient_4 % (2 ** check_index) == 0:
                                checked_quotient_4 = quotient_4 // (2 ** check_index)
                                if checked_quotient_4 % 2 != 0:
                                    return self.__RMB_UNIT[check_index]

        # 首先验证输入的数字是否合法
        p = regex.compile(r'^(\d*)(\.?)(\d*?)$')
        if not regex.match(p, num):
            raise NFLError(f"您输入的 {num} 不是一个有效的人民币金额")

        int_part, dot, decimal_part = regex.findall(p, num)[0]
        int_part = '0' if int_part == '' else int_part
        int_part = str(int(int_part))

        if len(int_part) > 16:
            raise NFLError(f"您输入的 {num} 人民币金额大到我有点找不到北")

        if dot != '' and len(decimal_part) > 2:
            raise NFLError(f"人民币金额只支持到分哦")

        upper_num = ''
        # 首先将数字转换成单个数字的列表
        num_list = [int(str_num) for str_num in str(int_part)]
        len_num = len(num_list)
        max_len = 2 ** (len(self.__RMB_UNIT) + 2)
        if len_num > max_len:
            raise NFLError('您输入的数字太长了，我只能支持' + str(max_len) + '位数字，嘤嘤嘤')
        for i in range(len_num):
            num_bit = num_list[i]

            i_index = len_num - i - 1
            # 根据算法计算出单位
            unit = cal_unit(i_index)
            # 如果首位数字为1并且为十位，则一可不写

            # 处理该位数字是0的情况
            if num_bit == 0:
                if self.__RMB_UNIT.count(unit) == 1:
                    """数字为0,但是该位数字的单位是CN_UNIT中的一个的情况\n
                        如果前一位是零，则删除零
                            这里会有一种极端情况，如果接下的超过四位数都为零，则会出现两个单位碰到一起\n
                            如果该单位小于等于原来的单位就在末尾用零代替第二个单位
                    """
                    if upper_num.endswith('零'):
                        upper_num = upper_num.rstrip('零')
                        # 默认单位长度为一，处理单位长度大于一的情况
                        stop = -2
                        last_unit = upper_num[-1:stop:-1][::-1]
                        while not ['拾', '佰', '仟'].count(last_unit) and not self.__RMB_UNIT.count(last_unit):
                            stop -= 1
                            last_unit = upper_num[-1:stop:-1][::-1]
                        if self.__RMB_UNIT.count(last_unit):
                            if self.__RMB_UNIT.index(unit) - self.__RMB_UNIT.index(last_unit) < 1:
                                upper_num += '零'
                                continue
                    upper_num += unit
                else:  # 如果数字为零并且不在上述单位，并且前一位不是零，则加上零
                    if not upper_num.endswith('零'):
                        upper_num += self.__RMB_NUM[num_bit]
            else:  # 数字不是零的情况就加上转换后的大写数字和单位
                if not (unit == '拾' and i == 0 and num_bit == 1):  # 若首位数字为1并且该位为十位就不需要一
                    upper_num += self.__RMB_NUM[num_bit]
                upper_num += unit
        if len(upper_num) > 1:
            upper_num = upper_num.rstrip('零')

        upper_num += '元'

        if dot == '' or decimal_part == '' or decimal_part == '0' or decimal_part == '00':
            upper_num += '整'
        else:
            if len(decimal_part) == 1:
                upper_num += self.__RMB_NUM[int(decimal_part)] + '角'
            else:
                jiao = decimal_part[0]
                fen = decimal_part[-1]
                if jiao == '0':
                    upper_num += '零'
                else:
                    upper_num += self.__RMB_NUM[int(jiao)] + '角'
                if fen != '0':
                    upper_num += self.__RMB_NUM[int(fen)] + '分'
            if int_part == '0':
                upper_num = upper_num[2::].removeprefix('零')

        return upper_num


def rmb2upper(num: str) -> str:
    """将输入的数字按照人民币大写的书写方式返回(支持16位)
        例如：
            123456789\n
            壹亿贰仟叁佰肆拾伍万陆仟柒佰捌拾玖元整\n

        :param num: 人民币小写
        :return: 返回人民币大写的形式
        """
    rmb2u = RMB2U()
    return rmb2u.rmb2upper(num)


if __name__ == '__main__':
    sn = '.0'
    # append_uint("牛逢路")
    # pop_unit()
    print(rmb2upper(str(sn)))
